package com.ycwl.basic.controller.mobile;

import com.github.pagehelper.PageInfo;
import com.ycwl.basic.constant.BaseContextHandler;
import com.ycwl.basic.model.mobile.order.OrderAppPageReq;
import com.ycwl.basic.model.mobile.order.RefundOrderReq;
import com.ycwl.basic.model.pc.order.req.OrderAddReq;
import com.ycwl.basic.model.pc.order.resp.OrderAppRespVO;
import com.ycwl.basic.model.wx.WxPayRespVO;
import com.ycwl.basic.service.pc.OrderService;
import com.ycwl.basic.utils.ApiResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Author：longbinbin
 * @Date：2024/12/4 17:16
 */
@RestController
@RequestMapping("/api/mobile/order/v1")
@Api(tags = "订单相关接口")
public class AppOrderController {

    @Autowired
    private OrderService orderService;

    @ApiOperation("用户端订单列表查询")
    @PostMapping("/page")
    public ApiResponse<PageInfo<OrderAppRespVO>> pageQuery(@RequestBody OrderAppPageReq orderReqQuery) {
        String userId = BaseContextHandler.getUserId();
        orderReqQuery.setMemberId(Long.valueOf(userId));
        return orderService.appPageQuery(orderReqQuery);
    }

    @ApiOperation("用户端订单详情查询")
    @GetMapping("getOrderDetails/{id}")
    public ApiResponse<OrderAppRespVO> getOrderDetails(@PathVariable("id") Long id) {
        return orderService.appDetail(id);
    }

    @ApiOperation("用户端订单新增")
    @PostMapping("/addOrder")
    public ApiResponse<WxPayRespVO> addOrder(@RequestBody OrderAddReq orderAddReq) throws Exception {
        return orderService.add(orderAddReq);
    }


    @ApiOperation("获取用户订单数量")
    @GetMapping("/getUserOrderCount")
    public ApiResponse<Integer> getUserOrderCount() {
        Long userId = Long.parseLong(BaseContextHandler.getUserId());

        return orderService.getOrderCountByUserId(userId);
    }

    @ApiOperation(value = "发起退款", notes = "发起退款")
    @PostMapping("/refundOrder")
    public ApiResponse<?> refundOrder(@RequestBody RefundOrderReq refundOrderReq) {
        return orderService.refundOrder(refundOrderReq);
    }

}
