package com.ycwl.basic.service.impl.pc;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ycwl.basic.enums.StatisticEnum;
import com.ycwl.basic.exception.BaseException;
import com.ycwl.basic.mapper.StatisticsMapper;
import com.ycwl.basic.mapper.FaceMapper;
import com.ycwl.basic.model.jwt.JwtInfo;
import com.ycwl.basic.model.mobile.face.FaceRecognizeResp;
import com.ycwl.basic.model.mobile.statistic.req.StatisticsRecordAddReq;
import com.ycwl.basic.model.pc.face.entity.FaceEntity;
import com.ycwl.basic.model.pc.face.req.FaceReqQuery;
import com.ycwl.basic.model.pc.face.resp.FaceRespVO;
import com.ycwl.basic.model.task.resp.SearchFaceRespVo;
import com.ycwl.basic.service.pc.FaceService;
import com.ycwl.basic.service.task.TaskFaceService;
import com.ycwl.basic.service.task.TaskService;
import com.ycwl.basic.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Author：longbinbin
 * @Date：2024/12/2 16:39
 */
@Slf4j
@Service
public class FaceServiceImpl implements FaceService {
    @Autowired
    private FaceMapper faceMapper;
    @Autowired
    private OssUtil ossUtil;
    @Autowired
    private TaskFaceService faceService;
    @Autowired
    private StatisticsMapper statisticsMapper;

    @Value("${face.score}")
    private float faceScore;
    @Autowired
    private TaskService taskTaskService;

    @Override
    public ApiResponse<PageInfo<FaceRespVO>> pageQuery(FaceReqQuery faceReqQuery) {
        PageHelper.startPage(faceReqQuery.getPageNum(),faceReqQuery.getPageSize());
        List<FaceRespVO> list = faceMapper.list(faceReqQuery);
        PageInfo<FaceRespVO> pageInfo = new PageInfo<>(list);
        return ApiResponse.success(pageInfo);
    }

    @Override
    public ApiResponse<List<FaceRespVO>> list(FaceReqQuery faceReqQuery) {
        return ApiResponse.success(faceMapper.list(faceReqQuery));
    }

    @Override
    public ApiResponse<FaceRespVO> getById(Long id) {
        return ApiResponse.success(faceMapper.getById(id));
    }

    @Override
    public ApiResponse<Integer> add(FaceEntity face) {
        face.setId(SnowFlakeUtil.getLongId());
        int i = faceMapper.add(face);
        if (i == 0) {
            return ApiResponse.fail("添加失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteById(Long id) {
        int i = faceMapper.deleteById(id);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> deleteByIds(List<Long> ids) {
        int i = faceMapper.deleteByIds(ids);
        if (i == 0) {
            return ApiResponse.fail("删除失败");
        }
        return ApiResponse.success(i);
    }

    @Override
    public ApiResponse<Integer> update(FaceEntity face) {
        int i = faceMapper.update(face);
        if (i == 0) {
            return ApiResponse.fail("修改失败");
        }
        return ApiResponse.success(i);
    }

    @Override
//    @Transactional(rollbackFor = Exception.class)
    public ApiResponse faceUPload(MultipartFile file,Long scenicId) {
        //获取用户id
        JwtInfo worker = JwtTokenUtil.getWorker();
        Long userId = worker.getUserId();
        log.info("当前登录用户信息：{}",worker);

        //1、上传人脸照片
        String faceUrl = uploadFileALiOss(file, userId);
        SearchFaceRespVo searchFaceRespVo = faceService.searchFace(scenicId, faceUrl);
        float score = searchFaceRespVo.getScore();
        if (score<faceScore) {
            //校验失败，删除，提示重新上传
            ossUtil.deleteFileByUrl(faceUrl);

            throw new BaseException("人脸照片校验失败，请重新上传");
        }
        // 2、查看人脸是否已上传
        FaceRespVO faceRespVO=faceMapper.getByMemberId(userId, scenicId);

        FaceEntity faceEntity = new FaceEntity();
        faceEntity.setScore(searchFaceRespVo.getScore());
        faceEntity.setMatchResult(searchFaceRespVo.getSearchResultJson());
        faceEntity.setMatchSampleIds(searchFaceRespVo.getSampleListIds().stream().map(String::valueOf).collect(Collectors.joining(",")));
        faceEntity.setCreateAt(new Date());
        faceEntity.setScenicId(scenicId);
        faceEntity.setMemberId(userId);
        faceEntity.setFaceUrl(faceUrl);
        if (faceRespVO==null) {
            //新增人脸
            faceEntity.setId(SnowFlakeUtil.getLongId());
            faceMapper.add(faceEntity);
            taskTaskService.autoCreateTaskByFaceId(faceEntity.getId());
        }else {
            //2、更新人脸
            faceEntity.setId(faceRespVO.getId());
            faceMapper.update(faceEntity);
            if (!Objects.equals(
                    faceRespVO.getMatchSampleIds(),
                    searchFaceRespVo.getSampleListIds().stream().map(String::valueOf).collect(Collectors.joining(","))
            )) {
                taskTaskService.autoCreateTaskByFaceId(faceEntity.getId());
            }
        }
        StatisticsRecordAddReq statisticsRecordAddReq = new StatisticsRecordAddReq();
        statisticsRecordAddReq.setMemberId(userId);
        statisticsRecordAddReq.setType(StatisticEnum.UPLOAD_FACE.code);
        statisticsRecordAddReq.setScenicId(scenicId);
        statisticsRecordAddReq.setMorphId(faceEntity.getId());
        statisticsMapper.addStatisticsRecord(statisticsRecordAddReq);
        FaceRecognizeResp resp = new FaceRecognizeResp();
        resp.setUrl(faceUrl);
        resp.setFaceId(faceEntity.getId());
        return ApiResponse.success(resp);
    }

    @Override
    public ApiResponse<FaceRespVO> getFaceByMemberId(Long memberId) {
        // TODO: 修改
        return ApiResponse.success(faceMapper.getByMemberId(memberId, 3928516560393736192L));
    }

    /**
     * 阿里oss图片上传
     *
     * @param file file
     * @param userId 用户id
     * @return 地址
     */
    private String uploadFileALiOss(MultipartFile file,Long userId)  {
        if (file.isEmpty()) {
            throw new RuntimeException("文件不存在！");
        }
        String originalFilename = file.getOriginalFilename();
        //获取文件名后缀
        String suffix = originalFilename.split("\\.")[1];
        if ("Jpeg".equals(suffix)) {
            suffix = "jpg";
        }
        //文件储存路径
        String filePath="";
        String dateStr = DateUtils.format(new Date(),"yyyy-MM-dd");

        filePath=filePath+dateStr+"/";
        // 生成文件名
        String fileName= userId+"." + suffix;
        InputStream inputStream ;
        try {
            inputStream = file.getInputStream();
        } catch (IOException e) {
            log.error("文件上传失败！", e);
            return null;
        }
        return ossUtil.uploadFile(inputStream,filePath+fileName) ;
    }
}
